<?php
namespace Zeedhi\DevOps\Model\Projectmanagement\Repositories;

use Doctrine\ORM\EntityRepository;
use Zeedhi\DevOps\Model\Util\LogicalDeleted;
use Zeedhi\DevOps\Util\Utilities;

class Budget extends EntityRepository {

    public function findIssueBudget($issueId, $proposalTitle = 'Proposta' ,$discountTitle = 'Desconto',
                                    $namePercent = 'Percentual') {
        $query =
            <<<SQL
           SELECT '  '||'$proposalTitle' AS TYPE_ORDER,
                    TO_CHAR(BUI.ID) AS BUDGET_ITEM_ID,
                    BUI.NAME AS ITEM_NAME,
                    NVL(BIB.QUANTITY, 0) AS PROPOSAL_QUANTITY,
                    TO_CHAR(NVL(BIB.ITEM_VALUE,NVL(BUI.VALUE, 0)),'999G999G999G990D00', 'NLS_NUMERIC_CHARACTERS = '',.''') AS ITEM_VALUE,
                    NVL(BIB.ITEM_VALUE,NVL(BUI.VALUE, 0)) AS ITEM_VALUE_TRUE,
                    NVL(BIB.ITEM_VALUE,NVL(BUI.VALUE, 0)) * NVL(BIB.QUANTITY, 0) AS PROPOSAL_VALUE,
                    BUI.UNIT_MEASURE,
                    ISS.ID AS ISSUE_ID,
                    BUD.ID,
                    BUD.EXTERNAL_ID
               FROM TASK ISS
               JOIN CLASSIFICATION_BUDGET_ITEM CBI ON (CBI.CLASSIFICATION = ISS.CLASSIFICATION )
               JOIN BUDGET_ITEM BUI ON (BUI.ID = CBI.BUDGET_ITEM)
               LEFT JOIN BUDGET BUD ON (BUD.TASK = ISS.ID)
               LEFT JOIN BUDGET_ITEM_BUDGET BIB ON (BIB.BUDGET = BUD.ID AND BIB.BUDGET_ITEM = BUI.ID)
              WHERE ISS.ID = :ISSUE_ID
              AND (BUI.STATUS = 'A' OR BIB.ID IS NOT NULL)

              UNION

            SELECT  ' '||'$discountTitle' AS TYPE_ORDER,
                    'XXXX' AS BUDGET_ITEM_ID,
                    ' ' AS ITEM_NAME,
                    NVL(NVL(BUD.DISCOUNT_PERCENT, 0) , 0) AS PROPOSAL_QUANTITY,
                    TO_CHAR(' ') AS ITEM_VALUE,
                    NVL(null, 0) AS ITEM_VALUE_TRUE,
                    NVL(BUD.DISCOUNT_VALUE, 0) AS PROPOSAL_VALUE,
                    '$namePercent' AS UNIT_MEASURE,
                    ISS.ID AS ISSUE_ID,
                    BUD.ID,
                    BUD.EXTERNAL_ID
               FROM TASK ISS
          LEFT JOIN BUDGET BUD ON (BUD.TASK   = ISS.ID )
              WHERE ISS.ID = :ISSUE_ID
           ORDER BY BUDGET_ITEM_ID

SQL;

        $params = array(
            "ISSUE_ID" => $issueId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }


    public function findManutIssueBudget($issueId) {
        $query =
            <<<SQL
              SELECT * FROM (
                    SELECT 
                    (sum( NVL(BIB.ITEM_VALUE,NVL(BUI.VALUE, 0)) * NVL(BIB.QUANTITY, 0))) AS TOTAL_VALUE, 1 as TYPE
               FROM TASK ISS
               JOIN CLASSIFICATION_BUDGET_ITEM CBI ON (CBI.CLASSIFICATION = ISS.CLASSIFICATION )
               JOIN BUDGET_ITEM BUI ON (BUI.ID = CBI.BUDGET_ITEM)
               LEFT JOIN BUDGET BUD ON (BUD.TASK = ISS.ID)
               LEFT JOIN BUDGET_ITEM_BUDGET BIB ON (BIB.BUDGET = BUD.ID AND BIB.BUDGET_ITEM = BUI.ID)
              WHERE ISS.ID = $issueId
              AND (BUI.STATUS = 'A' OR BIB.ID IS NOT NULL)
              AND (NVL(BIB.QUANTITY, 0) <> 0)
              

              UNION

            SELECT  
                    NVL(NVL(BUD.DISCOUNT_PERCENT, 0) , 0) AS TOTAL_VALUE, 2 as TYPE
                   
               FROM TASK ISS
          LEFT JOIN BUDGET BUD ON (BUD.TASK   = ISS.ID )
              WHERE ISS.ID = $issueId )
              ORDER BY TYPE 
              

             

SQL;
        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $result;
    }

    public function findManutIssueDiscount($issueId) {
        $query =
            <<<SQL
            SELECT MAINTENANCE_PERCENT, MAINTENANCE_VALUE FROM BUDGET BUD
              JOIN TASK ISS ON BUD.TASK = ISS.ID
              WHERE ISS.ID = $issueId
        

SQL;
        $result = $this->getEntityManager()->getConnection()->fetchAll($query);
        return $result;
    }

    public function findIssueBudgetPlots($issueId,$namePlots ='Parcela') {
        $query =
            <<<SQL
        SELECT  PPP.ORDER_PLOTS||'° $namePlots' AS ORDER_PLOTS,
                PPP.ORDER_PLOTS AS ORDER_PLOTS_NUM,
                PPP.BUDGET_ID,
                DECODE(PPP.VALUE_PLOTS,NULL,SUM(PPP.PROPOSAL_VALUE) + PPP.DISCOUNT_VALUE,PPP.VALUE_PLOTS) AS VALUE_PLOTS,
                PPP.DAYS_PAYMENT,PPP.DAYS_PAYMENT AS DAYS_PAYMENT_COPY,
                'Parcelas' AS GROUPBY,
                TO_CHAR(SUM(PPP.PROPOSAL_VALUE) + PPP.DISCOUNT_VALUE,'999G999G999G990D00', 'NLS_NUMERIC_CHARACTERS = '',.''') AS PROPOSAL_VALUE_VIEW,
                SUM(PPP.PROPOSAL_VALUE) + PPP.DISCOUNT_VALUE AS PROPOSAL_VALUE
        FROM (
        
        SELECT        
            DECODE(BPS.ID,NULL,1,BPS.ORDER_PLOTS) AS ORDER_PLOTS,
            DECODE(BPS.ID,NULL,NULL,BPS.VALUE_PLOTS) AS VALUE_PLOTS,
            DECODE(BPS.ID,NULL,30,BPS.DAYS_PAYMENT) AS DAYS_PAYMENT,
            BIB.ITEM_VALUE * BIB.QUANTITY AS PROPOSAL_VALUE,
            BUD.DISCOUNT_VALUE,BUD.ID AS BUDGET_ID
        FROM BUDGET BUD
        LEFT JOIN BUDGET_PLOTS BPS ON (BPS.BUDGET = BUD.ID)
        LEFT JOIN BUDGET_ITEM_BUDGET BIB ON (BIB.BUDGET = BUD.ID)
        WHERE BUD.TASK = :ISSUE_ID
        ) PPP 
        GROUP BY PPP.ORDER_PLOTS, PPP.VALUE_PLOTS, PPP.DAYS_PAYMENT, PPP.DISCOUNT_VALUE,PPP.BUDGET_ID        
        ORDER BY PPP.ORDER_PLOTS
        
SQL;

        $params = array(
            "ISSUE_ID" => $issueId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

    public function getProposalValues($issueId) {
        $query =
            <<<SQL
        SELECT SP.EXTERNAL_CODE AS SPONSOR,SP.NAME AS SPONSOR_NAME, RR.EXTERNAL_CODE AS REPORTER,
                NVL(OMP.ID, NVL(OPE.NRPRODUTO_ID,POM.ID)) AS OM_PRODUCT,PRO.NAME AS PRODUCT_NAME,RR.ORGANIZATION,
                NVL(OMP.ID, NVL(OPE.NRPRODUTO_ID,POM.ID)) AS OM_PRODUTO,ISS.DESCRIPTION,
                BUD.DISCOUNT_VALUE AS DISCOUNT_VALUE,BUD.DISCOUNT_PERCENT, RR.NAME AS REPORTER_NAME,
                RR.EMAIL AS REPORTER_EMAIL,BUD.ID AS BUDGET_ID,BUD.EXTERNAL_ID
            FROM TASK ISS
       LEFT JOIN PRODUCT PRO ON (PRO.PROJECT_ID = ISS.PROJECT)
       LEFT JOIN OM_PRODUTO POM ON (POM.NREXTERNAL_PRODUCT_ID = PRO.ID)
       LEFT JOIN ALLOCATION AR ON AR.TYPE = 'R' AND AR.TASK = ISS.ID
       LEFT JOIN ALLOCATION AL ON AL.TYPE = 'S' AND AL.TASK = ISS.ID
       LEFT JOIN RESOURCE_T RR ON AR.RESOURCE_T =  RR.ID
       LEFT JOIN RESOURCE_T SP ON AL.RESOURCE_T = SP.ID
       LEFT JOIN BUDGET BUD ON ISS.ID = BUD.TASK       
       LEFT JOIN CP_CALL CP ON CP.CALL_RELATIONSHIP = :ISSUE_ID AND CP.TYPE = 'I'
       LEFT JOIN PRODUCT PRO1 ON (PRO1.PROJECT_ID = CP.PRODUCT)
       LEFT JOIN OM_PRODUTO OMP ON (PRO1.ID = OMP.NREXTERNAL_PRODUCT_ID)
       LEFT JOIN OM_PRODUTO_ELEMENTO OPE ON (OPE.ID = ISS.MENU_ITEM_ID AND PRO.ID = OPE.NREXTERNAL_PRODUCT_ID)
           WHERE ISS.ID = :ISSUE_ID
SQL;

        $params = array(
            "ISSUE_ID" => $issueId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result[0];
    }

    public function getProposalItemValues($issueId) {
        $query =
            <<<SQL
             SELECT BIB.QUANTITY AS "quantity",
                    BIB.ITEM_VALUE AS "unitaryValue",
                    BIB.ITEM_VALUE * BIB.QUANTITY AS "totalValue",
                    BIB.UNIT_MEASURE AS "measurement",
                    ISS.ID AS ISSUE_ID,
                    BFI.CDSERVORCA AS "id"
               FROM TASK ISS
               JOIN BUDGET BUD ON (BUD.TASK = ISS.ID)
               JOIN BUDGET_ITEM_BUDGET BIB ON (BIB.BUDGET = BUD.ID)
               LEFT JOIN BUDGET_ITEM_FCT_ITEMORCAMENTO BFI ON (BFI.BUDGET_ITEM_ID = BIB.BUDGET_ITEM)
              WHERE ISS.ID = :ISSUE_ID
              AND (BIB.ITEM_VALUE IS NOT NULL)
SQL;

        $params = array(
            "ISSUE_ID" => $issueId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

    public function getUrlIntegrationSM($organizationId, $baseUri = '')
    {

        $query =
            <<<SQL
             SELECT DSURLSM||'$baseUri' AS URI
             FROM OM_INTEGRACOES
             WHERE NRORG = :ORGANIZATION_ID
SQL;

        $params = array(
            "ORGANIZATION_ID" => $organizationId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($result) ? null : $result[0]['URI'];
    }

    public function MaintenanceCost($organizationId)
    {

        $query =
            <<<SQL
              SELECT DISTINCT
               MA.VRPERCMANUCUSTOM AS MAINTENANCE_PERCENTAGE
          FROM (SELECT CDSERVORCA, NMSERVORCA
                  FROM FCT_SERVICOORCA
                 WHERE IDLICENUSOMANUT = 'L'
                   AND IDATIVO = 'S'
                   AND NRORG = :nrorg) LU,
               (SELECT FCT_SERVICOORCA.CDSERVORCA, FCT_SERVICOORCA.NMSERVORCA, 
                       FCT_SERVICOORCA.VRPERCMANUCUSTOM, FCT_SERVICOORCA.NRESCOPOMANUCUSTOM,
                       FCT_TIPOORCAMENTO.NMTIPOORCAMENTO
                  FROM FCT_SERVICOORCA
                  LEFT JOIN FCT_TIPOORCAMENTO
                    ON FCT_TIPOORCAMENTO.NRTIPOORCAMENTO = FCT_SERVICOORCA.NRESCOPOMANUCUSTOM
                   AND FCT_TIPOORCAMENTO.NRORG = FCT_SERVICOORCA.NRORG
                 WHERE FCT_SERVICOORCA.IDLICENUSOMANUT = 'M'
                   AND FCT_SERVICOORCA.IDATIVO = 'S'
                   AND FCT_SERVICOORCA.NRORG = :nrorg) MA,
               (SELECT CDTPMEDIDASERV, DSTPMEDIDASERV
                  FROM FCT_TIPOMEDIDASERV
                 WHERE IDATIVO = 'S'
                   AND NRORG = :nrorg) QT
SQL;

        $params = array(
            "nrorg" => $organizationId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($result) ? null : $result[0]['MAINTENANCE_PERCENTAGE'];
    }

    public function validateBudget($issueId) {
        $query =
            <<<SQL
            SELECT ISS.ID
                   FROM TASK ISS
                   JOIN CLASSIFICATION_BUDGET_ITEM CBI ON (CBI.CLASSIFICATION   = ISS.CLASSIFICATION )
                   JOIN BUDGET_ITEM BUI ON (BUI.ID = CBI.BUDGET_ITEM)
              LEFT JOIN BUDGET BUD ON (BUD.TASK = ISS.ID)
              LEFT JOIN BUDGET_ITEM_BUDGET BIB ON (BIB.BUDGET = BUD.ID AND BIB.BUDGET_ITEM = BUI.ID)
                  WHERE ISS.ID = :issueId
                    AND (BUI.STATUS = 'A' OR BIB.ID IS NOT NULL)
                    AND ROWNUM = 1
SQL;

        $params = array(
            "issueId" => $issueId
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($result) ? false : true;
    }

    public function getProposalValuesSoliserv($nrsoliserv) {
        $query =
            <<<SQL
              SELECT NRSEQORCAMENTO AS EXTERNAL_ID
                FROM FCT_ORCAMENTO
               WHERE NRSOLISERV = :nrsoliserv
                 AND IDATIVO = 'S'
                 AND IDSITUACAOORCAMENTO <> 'CANCELADO'
SQL;

        $params = array(
            "nrsoliserv" => $nrsoliserv
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($result) ? array() : $result[0];
    }

    public function validateAcceptProposalSoliservAlreadyAccepted($nrsoliserv) {
        $query =
            <<<SQL
              SELECT NRSEQORCAMENTO AS EXTERNAL_ID
                FROM FCT_ORCAMENTO
               WHERE NRSOLISERV = :nrsoliserv
                 AND IDATIVO = 'S'
                 AND IDSITUACAOORCAMENTO  = 'ACEITO'
SQL;

        $params = array(
            "nrsoliserv" => $nrsoliserv
        );
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($result) ? false : true;
    }

    public function findNrseqcontratoWithSoliservId($nrsoliserv) {
        $query =
            <<<SQL
             
                                           SELECT NRSEQCONTRATO
                                        FROM FCT_REFEORCACONT
                                        WHERE NRSEQORCAMENTO = ( SELECT NRSEQORCAMENTO
                                        FROM FCT_ORCAMENTO
                                       WHERE NRSOLISERV = :NRSOLISERV
                                         AND IDATIVO = 'S'
                                         AND IDSITUACAOORCAMENTO  = 'ACEITO')


SQL;

        $params = array(
            "NRSOLISERV" => $nrsoliserv
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);

    }


}
